#!/usr/bin/env python3
# -*- coding:utf-8 -*-

from flask import Flask, render_template, Response,request
from buildhat import SystemMediumMotor
from camera_pi import Camera
import time
import os, re

def execCmd(cmd):  
    r = os.popen(cmd)  
    text = r.read()  
    r.close()  
    return text  
txt = execCmd("ls /dev/video*")
detect_flag = re.search(r"video0", txt, flags=re.I)

app = Flask(__name__)

class Car(object):

    def __init__(self):
        #device need connected

        #The right rear motor
        self.Port_A=SystemMediumMotor('A')
        #The right front motor
        self.Port_B=SystemMediumMotor('B')
        #The left rear motor
        self.Port_C=SystemMediumMotor('C')
        #The left front motor
        self.Port_D=SystemMediumMotor('D')
    def set_spped(self,port,speed=100):
        port.start(int(speed))

    #   The following functions need to match the Raspberry Pi Build HAT port with the LEGO motor,
    #   otherwise it will not work properly

    def front(self,speed = 100,time_ms = 0):
        self.set_spped(self.Port_A,speed)
        self.set_spped(self.Port_B,speed)
        self.set_spped(self.Port_C,-speed)
        self.set_spped(self.Port_D,-speed)
        time.sleep(time_ms/1000)

    def rear(self,speed = 100,time_ms = 0):
        self.set_spped(self.Port_A,-speed)
        self.set_spped(self.Port_B,-speed)
        self.set_spped(self.Port_C,speed)
        self.set_spped(self.Port_D,speed)
        time.sleep(time_ms/1000)
    
    def right(self,speed = 100,time_ms = 0):
        self.set_spped(self.Port_A,-speed)
        self.set_spped(self.Port_B,speed)
        self.set_spped(self.Port_C,-speed)
        self.set_spped(self.Port_D,speed)
        time.sleep(time_ms/1000)

    def left(self,speed = 100,time_ms = 0):
        self.set_spped(self.Port_A,speed)
        self.set_spped(self.Port_B,-speed)
        self.set_spped(self.Port_C,speed)
        self.set_spped(self.Port_D,-speed)
        time.sleep(time_ms/1000)

    def front_left(self,speed = 100,time_ms = 0):
        self.set_spped(self.Port_A,speed)
        self.set_spped(self.Port_B,speed)
        self.set_spped(self.Port_C,-speed/5)
        self.set_spped(self.Port_D,-speed/5)
        time.sleep(time_ms/1000)

    def front_right(self,speed = 100,time_ms = 0):
        self.set_spped(self.Port_A,speed/5)
        self.set_spped(self.Port_B,speed/5)
        self.set_spped(self.Port_C,-speed)
        self.set_spped(self.Port_D,-speed)
        time.sleep(time_ms/1000)

    def rear_left(self,speed = 100,time_ms = 0):
        self.front_left(-speed,time_ms)

    def rear_right(self,speed = 100,time_ms = 0): 
        self.front_right(-speed,time_ms)
        
    def stop(self):
        self.Port_A.stop()
        self.Port_B.stop()
        self.Port_C.stop()
        self.Port_D.stop()

def main(status):
    if status == "ic_Up":
        car.front()
    elif status == "ic_left":
        car.left()
    elif status == "ic_right":
        car.right()
    elif status == "ic_Down":
        car.rear()
    elif status == "ic_stop":
        car.stop() 
    elif status == "stop":
        car.stop() 
    elif status == "ic_leftUp":
        car.front_left() 
    elif status == "ic_rightUp":
        car.front_right()   
    elif status == "ic_leftDown":
        car.rear_left() 
    elif status == "ic_rightDown":
        car.rear_right()  
    print(status)

@app.route('/')
def index():
    """Video streaming home page."""
    if(detect_flag != None  ):
        return render_template('index.html')
    else:
        return render_template('index2.html')

@app.route("/cmd", methods=['GET', 'POST'])
def button():
    if request.method == 'POST':
        data = request.form.to_dict()
        main(data['id'])
    return render_template('index.html')
#Automatically detects whether the camera exists and switches modes
if(detect_flag != None  ):
    def gen(camera):
        """Video streaming generator function."""
        yield b'--frame\r\n'
        while True:
            frame = camera.get_frame()
            yield b'Content-Type: image/jpeg\r\n\r\n' + frame + b'\r\n--frame\r\n'
    @app.route('/video_feed')
    def video_feed():
        """Video streaming route. Put this in the src attribute of an img tag."""
        return Response(gen(Camera()),
                        mimetype='multipart/x-mixed-replace; boundary=frame')


if __name__ == '__main__':
    os.system("/etc/rc.local")
    time.sleep(1)
    car = Car()
    car.stop()

    app.run(host='0.0.0.0', threaded=True,debug = False)

